#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "defines.h"
#include "video_low.h"
#include "time.h"

/* defines */

#define  SWAP16(data)    ((U16) (data >> 8) | (data << 8))

/* external variables */

/* None */

/* global variables */

/* None */

/* local variables */

/* None */

/* local prototypes */

/* None */

/*
===============================================================================
 \brief Displays a BMP image to screen
 \param FileName: file name of image to load from disk
 \param Scaling: Scale image by the passed scaling factor
 \return if we can display the image return 0 else we fail return 1
===============================================================================
*/
U8 LoadBMP(const char *FileName, U32 Scaling)
{
    FILE *FilePointer = NULL;
    COLOUR_TYPE    Colour;
    U32 FileSize = 0;
    U8 *pBuffer = NULL;
    U32 Result;
    BITMAP_FILE_HEADER_TYPE Header1;
    DIB_FILE_HEADER_TYPE Header2;
    U32 index;
    U32 Padding;
    U8 BytesPerPixel;
    
    /* display text in red */
    Colour.r = 255;
    Colour.g = 0;
    Colour.b = 0;
    
    FilePointer = fopen(FileName,"rb");
    
    /* check for valid file */
    if(FilePointer == NULL)
    {
        MyPrint("cant open file", 0, 0, SCALING);
        Redraw();
        DelayMs(2000);
        return 1;               
    }
    
    /* obtain file size: */
    fseek(FilePointer , 0 , SEEK_END);
    FileSize = ftell (FilePointer);
    rewind(FilePointer);
  
    if(FileSize < 14)
    {
        MyPrint("invalid bmp file size", 0, 0, SCALING);
        fclose(FilePointer);        
        Redraw();
        DelayMs(2000);
        return 1;                 
    }
    
    /* allocate memory to contain the whole file: */
    pBuffer = (U8 *) malloc (sizeof(U8)*FileSize);
  
    if(pBuffer == NULL)
    {
        MyPrint("cant malloc memory for file", 0, 0, SCALING);
        fclose(FilePointer);        
        Redraw();
        DelayMs(2000);
        return 1;
    }
    
    /* copy the file into the buffer: */
    Result = fread(pBuffer,1,FileSize,FilePointer);
  
    /* did we read the whole file ok ?*/
    if (Result != FileSize) 
    {
        MyPrint("read file error !", 0, 0, SCALING);
        fclose(FilePointer);
        free(pBuffer);        
        Redraw();
        DelayMs(2000);
        return 1;
    }
    
    /* file in memory so we dont need the handle anymore */
    fclose(FilePointer);
    
    /* fill up bmp header data*/
    memcpy(&Header1.MagicNumber,pBuffer,sizeof(U16));
    index = sizeof(U16);
    Header1.MagicNumber = SWAP16(Header1.MagicNumber);
  
    memcpy(&Header1.Size,&pBuffer[index],sizeof(U32));
    index += sizeof(U32);
  
    memcpy(&Header1.Reserved,&pBuffer[index],sizeof(U32));
    index += sizeof(U32);
  
    memcpy(&Header1.Offset,&pBuffer[index],sizeof(U32));
    index += sizeof(U32);
  
    /* fill the dib header */
    memcpy(&Header2,&pBuffer[index],sizeof(Header2));
  
    /* put pointer at the end of buffer */
    pBuffer += Header2.ImageSize + Header1.Offset; 
  
    BytesPerPixel = (Header2.BitsPerPixel / 8);
  
    /* work out the amount of padding per line */
    Padding = Header2.ImageSize - (Header2.BitMapHeight * Header2.BitMapWidth * BytesPerPixel);
    Padding /= Header2.BitMapHeight;
    
    DrawBmp(pBuffer, Header2.BitMapWidth, Header2.BitMapHeight, BytesPerPixel, Padding, Scaling);
    
    free(pBuffer);
    return 0;                    
}
