#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "defines.h"
#include "time.h"
#include "inputs.h"
#include "video_high.h"

/* defines */

#define SKILL_LEVEL    "Select a skill level (1-4): " 

/* external variables */

extern U16 CursorLocX;
extern U16 CursorLocY;
extern GAMESTATE_TYPE GameState;

/* global variables */

/* None */

/* local variables */

/* None */

/* local prototypes */

static U8 GetSelection(const char *String);

/*
===============================================================================
 \brief Gets selected game skill level from user
 \param None
 \return returns 1 if user wants to exit game lese zero to continou
===============================================================================
*/
U8 GetSkillLevel(void)
{
    U8 Skill;
    U8 Exit = 0;                  
                      
    Skill = GetSelection(SKILL_LEVEL);
    
    switch(Skill)
    {
        case 0:
            /* User chose to exit game */ 
            Exit = 1;
        break;
             
        case 1:
            GameState.SkillLevel = Easy;     
        break;
             
        case 2:
            GameState.SkillLevel = Medium; 
        break;
        
        case 3:
            GameState.SkillLevel = Hard; 
        break;
             
        case 4:
            GameState.SkillLevel = Crazy;      
        break;
        
        default:
            /* do nothing */        
        break;             
    }
    
    return Exit;     
}

/*
===============================================================================
 \brief Displays a question to the user and returns the response
 \param string: message to print to the screen
 \return users response to the question passed
===============================================================================
*/
static U8 GetSelection(const char *String)
{
    U16 X;
    U16 Y;
    U8 UserSelection = 0;                  
    INPUT_TYPE UserControls;
    memset(&UserControls,0,sizeof(UserControls));                   

    /* draw skill selection window */
    Message(String); 
    Redraw();
    
    X = CursorLocX;
    Y = CursorLocY;
     
    /* flash cursor until user selects a skill level */
    do
    {
        MyPrint("_", X, Y, SCALING);
        Redraw();
        DelayMs(50);
        MyPrint(" ", X, Y, SCALING);
        Redraw();
        DelayMs(50);
        GetUserInput(&UserControls);
                         
    }while((UserControls.One == 0) && (UserControls.Two == 0) 
            && (UserControls.Three == 0) && (UserControls.Four == 0) && (UserControls.Exit == 0));
    
    /* if user didnt exit they selected a skill level */        
    if(UserControls.Exit == 0)
    {
        /* so record the choosen skill level */
        if(UserControls.One == 1)
        {
            UserSelection = 1;
            MyPrint("1", X, Y, SCALING);
            Redraw();
        }
        
        if(UserControls.Two == 1)
        {
            UserSelection = 2;
            MyPrint("2", X, Y, SCALING);
            Redraw();
        }
        
        if(UserControls.Three == 1)
        {
            UserSelection = 3;
            MyPrint("3", X, Y, SCALING);
            Redraw();
        }
        
        if(UserControls.Four == 1)
        {
            UserSelection = 4;
            MyPrint("4", X, Y, SCALING);
            Redraw();
        }
    }
    
    return UserSelection; 
}
