#include <stdio.h>
#include <stdlib.h>
#include "types.h"
#include "defines.h"

/* defines */

#define BASEMOVE        35
#define ANGLESCALE      20
#define MOVESCALE       150l
#define PLAYERSIZE      0x5800l     /* player radius */

/* external variables */

extern FixedPoint Sintable[ANGLES+ANGLES/4];
extern U8 Map[MAP_HEIGHT][MAP_WIDTH];

/* global variables */

PL_POINT_TYPE Player;

/* local variables */

static S16   Anglefrac = 0;
static FixedPoint *Costable = Sintable+(ANGLES/4);

/* local prototypes */

static void Thrust(S32 angle, S32 speed);
static void ClipMove(S32 xmove, S32 ymove);
static U8 TryMove(void);

/* 
================================================================
=
= Function: MovePlayer
=
= Description:
=
= check to see if user has pressed the arrow keys if so it
= moves the player in the direction the user wants.
=
================================================================
*/
void MovePlayer(INPUT_TYPE *Controls)
{
    S32    angle; 
    S32    angleunits;
  
    /* rotate to the right */
    if(Controls->Right != 0)
    {
        Anglefrac += BASEMOVE;
        angleunits = Anglefrac / ANGLESCALE;
        Anglefrac -= angleunits * ANGLESCALE;
        Player.Angle -= angleunits;               
        
        if(Player.Angle < 0)
        {
            Player.Angle += ANGLES;
        }
       
    }

    /* rotate to the left */
    if(Controls->Left != 0)
    {
        Anglefrac -= BASEMOVE;
        angleunits = Anglefrac / ANGLESCALE;
        Anglefrac -= angleunits * ANGLESCALE;
        Player.Angle -= angleunits;               
        
        if(Player.Angle >= ANGLES)
        {
            Player.Angle -= ANGLES;
        }
    }
    
    /* move forward */
    if(Controls->Forward != 0)
    {
        Thrust(Player.Angle,BASEMOVE*MOVESCALE);                
    }
    
    /* move back */
    if(Controls->Back != 0)
    {   
        /* flip angle so new x , y coord's are behind player */
        angle = Player.Angle + ANGLES/2;
        
        if (angle >= ANGLES)
        {
            angle -= ANGLES;
        }
        
        Thrust(angle,BASEMOVE*MOVESCALE);                
    }
    
}

/* 
================================================================
=
= Function: Thrust
=
= Description:
=
= calculates new speed and direction of player 
=
================================================================ 
*/
static void Thrust(S32 angle, S32 speed)
{
    S32 xmove;
    S32 ymove;
    
    /* calculate new x and y positions of player*/
    xmove = FixedMul(speed,Costable[angle]);
    ymove = -FixedMul(speed,Sintable[angle]);
    
    /* check to see if new positions are valid on map*/
    ClipMove(xmove,ymove);    
}

/* 
================================================================
=
= Function: ClipMove
=
= Description:
=
= move player in the direction the user wants if there are no 
= walls present at the new corrd's
=
================================================================ 
*/
static void ClipMove(S32 xmove, S32 ymove)
{
    S32    basex;
    S32    basey;

    /* store current position */
    basex = Player.X;
    basey = Player.Y;
    
    /* add new position to current one */
    Player.X = basex + xmove;
    Player.Y = basey + ymove;
    
    /* check if move is ok */
    if(TryMove() == 1)
    {
        return;
    }
    
    /* try moving in the x direction only */
    Player.X = basex + xmove;
    Player.Y = basey;
    
    if(TryMove ())
    {
        return;
    }
    
    /* try moving in the y direction only */
    Player.X = basex;
    Player.Y = basey+ymove;
    
    if (TryMove())
    {
        return;
    }
    
    /* no luck stay in current location */
    Player.X = basex;
    Player.Y = basey;
}

/*
===============================================================================
 \brief Checks to see if player can move 
 \param None
 \return true if move ok
===============================================================================
*/
static U8 TryMove(void)
{
    S32    xl;
    S32    yl;
    S32    xh;
    S32    yh;
    S32    x;
    S32    y;
    U8     check;

    /* work out players max x y size in grid coord's*/
    xl = (Player.X - PLAYERSIZE) >> MAP_TO_GRID_SHIFT;
    yl = (Player.Y - PLAYERSIZE) >> MAP_TO_GRID_SHIFT;

    xh = (Player.X + PLAYERSIZE) >> MAP_TO_GRID_SHIFT;
    yh = (Player.Y + PLAYERSIZE) >> MAP_TO_GRID_SHIFT;
    
    /* check for solid walls */
    for(y = yl ; y <= yh; y++)
    {
        for(x = xl; x <= xh; x++)
        {
            check = Map[y][x];
            
            if(check != 0)
            {
                return 0;
            }
        }
    }
    
    return 1;
}
