#ifndef TYPES_H
#define TYPES_H

typedef unsigned char   U8;
typedef signed char     S8;
typedef unsigned short  U16;
typedef signed short    S16;
typedef unsigned int    U32;
typedef signed int      S32;
typedef S32             FixedPoint;    /* fixed point 16:16 */

typedef enum Skill
{
    Easy = 0,
    Medium,
    Hard,
    Crazy        
} SKILL_TYPE; 

typedef enum Graphics
{
    InterRimPic1 = 0,    
    StatusBarPic,
    ColonPic,
    NumZeroPic,
    NumOnePic,
    NumTwoPic,
    NumThreePic,
    NumFourPic,
    NumFivePic,
    NumSixPic,
    NumSevenPic,
    NumEightPic,
    NumNinePic,
    SCPPic,
    NumberPics
}GRAPHICS_TYPE;

typedef enum Level
{
    Level1 = 0,
    Level2,
    Level3,
    Level4        
} LEVEL_TYPE;

typedef enum Play
{
    Playing = 0, 
    Died,
    Victory               
} PLAY_STATE_TYPE;

typedef struct pic{
  U32 	 Width;
  U32 	 Height;
  U32 	 BytesPerPixel; /* 2:RGB16, 3:RGB, 4:RGBA */ 
  U8	 *PixelData;
} PIC_TYPE;

typedef struct colorRGB
{
    S32 r;
    S32 g;
    S32 b;
    S32 a;
} COLOUR_TYPE;

typedef struct bmp_header
{
    U16 MagicNumber;
    U32 Size;
    U32 Reserved;
    U32 Offset;        
} BITMAP_FILE_HEADER_TYPE;

typedef struct dib_header
{
    U32 HeaderSize;
    U32 BitMapWidth;
    U32 BitMapHeight;
    U16 ColourPlanes;
    U16 BitsPerPixel;
    U32 Compression;
    U32 ImageSize;
    S32 HorizontalRes;
    S32 VerticalRes;
    U32 ColoursInPal;
    U32 ImportantColours;
    
} DIB_FILE_HEADER_TYPE;

typedef struct input
{
    U8 Left;
    U8 Right;
    U8 Forward;
    U8 Back;
    U8 Exit;
    U8 One;
    U8 Two;
    U8 Three;
    U8 Four;
}INPUT_TYPE;

typedef struct game
{
    SKILL_TYPE SkillLevel;
    LEVEL_TYPE LevelNum;
    PLAY_STATE_TYPE PlayState;
    U8 Health;
    U8 Lives;
    U8 Scp;
    S32 TimeCount;            
}GAMESTATE_TYPE;

typedef struct PlayersView{
    S32 X;
    S32 Y;
    S16 Angle;
} PL_POINT_TYPE;

typedef struct point{
    S16 x;
    S16 y;
} POINT_TYPE;

typedef struct statobj
{
    U8        TileX;
    U8        TileY;
    S16       ShapeNum;    /* if shapenum == -1 the obj has been removed */
    U8        *VisableSpot;        
} STATIC_OBJECT_TYPE;

typedef struct visobj
{
    S16    ViewX;
    S16    ViewHeight;
    S16    ShapeNum;            
} VISABLE_OBJECT_TYPE;   

#endif
